////////////////////////////////////////////////////////////////////////////////
// auls_rendermask.cpp
// o̓}XNvOC
////////////////////////////////////////////////////////////////////////////////

#include <aulslib/generic.h>
#include <aulslib/aviutl.h>
#include <aulslib/exedit.h>
#include <aulslib/memref.h>

using namespace auls;

#include <vector>
#include <algorithm>

static const LPSTR FILTER_NAME = "o̓}XN";
static const LPSTR FILTER_INFORMATION = "Aulso̓}XN";

struct OBJECT_RANGE {
	int begin_frame;
	int end_frame;

	bool operator==(int i)
	{
		return i >= begin_frame && i <= end_frame;
	}

	bool operator<(const OBJECT_RANGE& r)
	{
		return this->begin_frame < r.begin_frame;
	}
};



int g_filter_index;

EXEDIT_FILTER **g_static_filter_table;
EXEDIT_FILTER **g_loaded_filter_table;
EXEDIT_OBJECT **g_sorted_object_table;
int *g_scene_displaying; // ̃V[ɔzuIuWFNgoȂ悤ɂ邽߂ɕ\V[擾Kv

std::vector<OBJECT_RANGE> g_range_vec;

int g_frame_output; // o̓t[
                    // ̒l fpip->frame ɑĊgҏW̎st[ωB


int g_prev_select_begin; // o͏IɑI͈͂ɖ߂߂ɁA
int g_prev_select_end;   // I͈̓t[ۑϐB

BOOL func_init(FILTER *fp)
{
	auls::CMemref memref;

	// Q
	if(!memref.Init(fp)) goto memref_failure;
	g_static_filter_table = memref.Exedit_StaticFilterTable();
	g_loaded_filter_table = memref.Exedit_LoadedFilterTable();
	g_sorted_object_table = memref.Exedit_SortedObjectTable();
	g_scene_displaying    = memref.Exedit_SceneDisplaying();
	if(!g_static_filter_table || !g_loaded_filter_table || !g_sorted_object_table || !g_scene_displaying) goto memref_failure;

	if(0) {
		memref_failure:
		MessageBox(auls::AviUtl_GetWindow(fp), memref.GetErrorString(), FILTER_INFORMATION, MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}



	return TRUE;
}

BOOL func_proc(FILTER *fp, FILTER_PROC_INFO *fpip)
{
	if(!fp->exfunc->is_saving(fpip->editp)) return TRUE;
	for(int i = 0; i < (int)g_range_vec.size(); i++) {
		OBJECT_RANGE &r = g_range_vec[i];
		if(g_frame_output == r.begin_frame) {
			g_frame_output = r.end_frame+1;
			break;
		}
	}
	fpip->frame = g_frame_output++;
	return TRUE;
}

BOOL func_update(FILTER *fp, int status)
{
	static bool init = false;

	if(!init && status == FILTER_UPDATE_STATUS_ALL) {
		// ̃tB^̃CfbNX擾
		int i=0;
		while(g_static_filter_table[i]) i++;
		while(g_loaded_filter_table[i]) {
			if(!strcmp(g_loaded_filter_table[i]->name, FILTER_NAME)) {
				g_filter_index = i;
				break;
			}
			i++;
		}
		init = true;
	}

	return TRUE;
}

BOOL func_save_start(FILTER *fp, int s, int e, void *editp)
{
	g_prev_select_begin = s;
	g_prev_select_end   = e;
	g_frame_output = s;

	// ̃tB^̔zuꂽ͈͂擾
	for(EXEDIT_OBJECT** p = g_sorted_object_table; *p; p++) {
		if((*p)->scene_set == *g_scene_displaying && (*p)->filter_param[0].id == g_filter_index) {
			OBJECT_RANGE r = { Range((*p)->frame_begin, s, e), Range((*p)->frame_end, s, e) };
			g_range_vec.push_back(r);
		}
	}
	std::sort(g_range_vec.begin(), g_range_vec.end());

	// dIuWFNg𓝍
	int i = 0, n = g_range_vec.size()-1;
	while(i < n) {
		OBJECT_RANGE &r1 = g_range_vec[i], &r2 = g_range_vec[i+1];
		if(r1.end_frame+1 >= r2.begin_frame) {
			if(r1.end_frame+1 >= r2.end_frame) { // IuWFNg͈͂SɊ܂܂
				g_range_vec.erase(g_range_vec.begin()+i+1);
			} else {
				r1.end_frame = r2.end_frame;
				g_range_vec.erase(g_range_vec.begin()+i+1);
			}
			n--;
		} else {
			i++;
		}
	}

	// o̓t[vZ
	{
		int len = e-s;
		int n = g_range_vec.size();
		for(int i = 0; i < n; i++) {
			OBJECT_RANGE &r = g_range_vec[i];
			len -= r.end_frame - r.begin_frame + 1;
		}
		fp->exfunc->set_select_frame(editp, s, s+len);
	}
	return TRUE;
}

BOOL func_save_end(FILTER *fp, void *editp)
{
	fp->exfunc->set_select_frame(editp, g_prev_select_begin, g_prev_select_end);
	g_range_vec.clear();
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
// tB^
////////////////////////////////////////////////////////////////////////////////

static const LPSTR FILTER2_NAME = "o̓}XN()";
static const LPSTR FILTER2_INFORMATION = "Aulso̓}XN()";

BOOL func_proc2(FILTER *fp, FILTER_PROC_INFO *fpip)
{
	if(!fp->exfunc->is_saving(fpip->editp)) return TRUE;
	fpip->frame = g_frame_output;
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
// tB^o^
////////////////////////////////////////////////////////////////////////////////

FILTER_DLL filter = {
	FILTER_FLAG_ALWAYS_ACTIVE | FILTER_FLAG_NO_CONFIG | FILTER_FLAG_EX_INFORMATION,
	0, 0,
	FILTER_NAME,
	0, NULL, NULL, NULL, NULL,
	0, NULL, NULL,
	func_proc,
	func_init,
	NULL,
	func_update,
	NULL,
	NULL,
	NULL,
	NULL,
	0,
	FILTER_INFORMATION,
	func_save_start,
	func_save_end,
};

FILTER_DLL filter2 = {
	FILTER_FLAG_AUDIO_FILTER | FILTER_FLAG_ALWAYS_ACTIVE | FILTER_FLAG_NO_CONFIG | FILTER_FLAG_EX_INFORMATION,
	0, 0,
	FILTER2_NAME,
	65, NULL, NULL, NULL, NULL, // gҏW^CCɔzułȂ悤ɃgbNo[̐ݒ
	0, NULL, NULL,
	func_proc2,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	0,
	FILTER2_INFORMATION,
};

FILTER_DLL *table[] = { &filter, &filter2, NULL };

extern "C" __declspec(dllexport) FILTER_DLL **GetFilterTableList(void)
{
	return table;
}
